/*******************************************************************************
*23456789 123456789 123456789 123456789 123456789 123456789 123456789 123456789
*       10        20        30        40        50        60        70        80
*
* count-down - An animated count-down rendition drawn with cairo looking like
* the one seen in this video http://www.youtube.com/watch?v=Fuy2W6gY7hQ and
* intended to be used in the kazam screenrecorder
*
* rendering.c - all cairo-related drawing functions
*
* Copyright 2012 Mirco Müller
*
* Authors:
*    Mirco "MacSlow" Müller <macslow@gmail.com>
*
* This program is free software: you can redistribute it and/or modify it
* under the terms of the GNU General Public License version 3, as published
* by the Free Software Foundation.
*
* This program is distributed in the hope that it will be useful, but
* WITHOUT ANY WARRANTY; without even the implied warranties of
* MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
* PURPOSE.  See the GNU General Public License for more details.
*
* You should have received a copy of the GNU General Public License along
* with this program.  If not, see <http://www.gnu.org/licenses/>.
*
*******************************************************************************/

#include <glib.h>
#include <math.h>
#include <pango/pango.h>

#include "rendering.h"

void
_draw_number (cairo_t* cr,
			  gdouble  width,
			  gdouble  height,
			  gint     number)
{
	/* create pango desc/layout */
	PangoLayout* layout = pango_cairo_create_layout (cr);
	PangoFontDescription* desc = pango_font_description_from_string ("Ubuntu");

	pango_font_description_set_absolute_size (desc, 0.75 * height * PANGO_SCALE);

    /* set some desired font-attributes */
    pango_font_description_set_weight (desc, PANGO_WEIGHT_NORMAL);
    pango_font_description_set_style (desc, PANGO_STYLE_NORMAL);
    pango_layout_set_font_description (layout, desc);
    pango_font_description_free (desc);

	/* print and layout string (pango-wise)*/
	GString* string = g_string_new ("");
	g_string_printf (string, "%d", number);
	pango_layout_set_text (layout, string->str, -1);
	g_string_free (string, TRUE);

	PangoRectangle inkRect     = {0, 0, 0, 0};
    PangoRectangle logicalRect = {0, 0, 0, 0};
	pango_layout_get_extents (layout, &inkRect, &logicalRect);

    gdouble x = width / 2 - inkRect.x / PANGO_SCALE - inkRect.width / PANGO_SCALE / 2;
	gdouble y = height / 2 - inkRect.y / PANGO_SCALE - inkRect.height / PANGO_SCALE / 2;
    
	/* draw text */
	cairo_move_to (cr, x, y);
	pango_cairo_layout_path (cr, layout);
    cairo_set_source_rgba (cr, 0.0, 0.0, 0.0, 1.0);
	cairo_fill (cr);

	/* clean up */
	g_object_unref (layout);
}

void
render (cairo_t* cr,
		gint     width,
		gint     height,
		gdouble  angle,
		gint     number)
{
	cairo_save (cr);

	/* clear context */
	cairo_scale (cr, width, height);
	cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
	cairo_paint (cr);
	cairo_set_operator (cr, CAIRO_OPERATOR_SOURCE);
	cairo_set_source_rgba (cr, 0.9, 0.9, 0.9, 0.7);
	cairo_paint (cr);

	gdouble strokeRadian = G_PI / 180.0 * (angle - 180.0);
	gdouble fillRadian = -G_PI / 180.0 * (angle + 90.0);
	gdouble x = sin (strokeRadian);
	gdouble y = cos (strokeRadian);

	/* draw "tinted" area */
	cairo_set_source_rgba (cr, 0.65, 0.65, 0.65, 0.9);
	cairo_move_to (cr, 0.5, 0.5);
	cairo_arc (cr, 0.5, 0.5, 0.75, G_PI / 180.0 * 270.0, fillRadian);
	cairo_close_path (cr);
	cairo_fill (cr);

	/* draw black "grid" */
	cairo_set_line_join (cr, CAIRO_LINE_JOIN_ROUND);
	cairo_set_line_cap (cr, CAIRO_LINE_CAP_ROUND);
	cairo_set_line_width (cr, 0.0025);
	cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
	cairo_move_to (cr, 0.0, 0.5);
	cairo_line_to (cr, 1.0, 0.5);
	cairo_move_to (cr, 0.5, 0.0);
	cairo_line_to (cr, 0.5, 1.0);
	cairo_stroke (cr);

	/* draw two thick black lines */
	cairo_set_line_width (cr, 0.0125);
	cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
	cairo_move_to (cr, 0.5, 0.0);
	cairo_line_to (cr, 0.5, 0.5);
	cairo_rel_line_to (cr, x, y);
	cairo_stroke (cr);

	/* draw two white circles */
	cairo_set_line_width (cr, 0.0075);
	cairo_set_source_rgb (cr, 1.0, 1.0, 1.0);
	cairo_arc (cr, 0.5, 0.5, 0.4, 0.0, 2 * G_PI);
	cairo_stroke (cr);
	cairo_arc (cr, 0.5, 0.5, 0.35, 0.0, 2 * G_PI);
	cairo_stroke (cr);

	cairo_restore (cr);

	/* draw number */
	cairo_set_source_rgb (cr, 0.0, 0.0, 0.0);
	_draw_number (cr, width, height, number);
}
